//---------------------------------------------------------------------------
#include <vcl.h>
#include <math.h>
#include <stdio.h>
#include "Unit1.h"
#include "graph.h"
#include "code_parser.h"
#include "bliepdsp_sdl.h"
#include "settings.h"
#include "about.h"

#pragma hdrstop

#define GRAPH_WIDTH 600

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

Tmainform *mainform;

int startdrag;
bool klik = false;
typedef enum dragdir { left=0, right=1 } dragdir;
typedef enum grafiek { move=0, zoom=1, select=2 } grafiek;
grafiek mousestate;
dragdir dir;

//---------------------------------------------------------------------------
__fastcall Tmainform::Tmainform(TComponent* Owner)
        : TForm(Owner), scrollspeed(20) {

  this->DoubleBuffered = true;
  Screen->Cursors[1] = LoadCursorFromFile("move.cur");
  Screen->Cursors[2] = LoadCursorFromFile("zoom.cur");
  graphcanvas->Cursor = (TCursor)1;
}
//---------------------------------------------------------------------------

void __fastcall Tmainform::Open_fileClick(TObject *Sender) {

  OpenDialog1->Filter = "*.wav|*.wav|*.ogg|*.ogg|*.riff|*.riff|*.voc|*.voc|*.aiff|*.aiff";
  OpenDialog1->Execute();

  if (OpenDialog1->FileName != NULL) {
    if (wave1.data != NULL)
      wave1.freedata();
    if (wave2.data != NULL)
      wave1.freedata();

  if (ExtractFileExt(OpenDialog1->FileName) == ".wav")
  {
    wave1.read_wav(OpenDialog1->FileName.c_str());
    wave2.read_wav(OpenDialog1->FileName.c_str());
  } else {
    wave1 = bliep_decode_file(OpenDialog1->FileName.c_str());
    wave2 = wave1;
    wave2.data = (char *) malloc(wave2.header.data_size);
    if (wave2.data != NULL)
      memcpy(wave2.data, wave1.data, wave1.header.data_size);
  }


    //Duration
    label_position->Caption = "00:00 / " + wave1.get_duration();
    label_duration->Caption = wave1.get_duration() + " minutes";

    //Audioformat
    if (wave1.header.audio_format == 1)
      label_audioformat->Caption = "PCM (No compression)";
    else
      label_audioformat->Caption = "Some form of compression";

    //Numchannels
    if (wave1.header.num_channels == 1)
      label_numchannels->Caption = "1 (Mono)";
    else if (wave1.header.num_channels == 2)
      label_numchannels->Caption = "2 (Stereo)";
    else
      label_numchannels->Caption = wave1.header.num_channels;

    //Samplerate
    label_samplerate->Caption = AnsiString(wave1.header.sample_rate);

    //Byterate
    label_byterate->Caption = AnsiString(wave1.header.byte_rate);

    //Blockalign
    label_blockalign->Caption = AnsiString(wave1.header.block_align);

    //Bitspersample
    label_bitspersample->Caption = AnsiString(wave1.header.bits_per_sample) + " bits";

    graph.clear();
    graph.data(&wave1);
    graph.draw(0,wave1.num_samples);
  }
}

//---------------------------------------------------------------------------
void __fastcall Tmainform::FormClose(TObject *Sender, TCloseAction &Action) {

  wave1.freedata();
  wave2.freedata();
}
//---------------------------------------------------------------------------

void __fastcall Tmainform::SpeedButton1Click(TObject *Sender) {

  mousestate = move;
  graphcanvas->Cursor = (TCursor)1;
}
//---------------------------------------------------------------------------

void __fastcall Tmainform::SpeedButton2Click(TObject *Sender) {

  mousestate = zoom;
  graphcanvas->Cursor = (TCursor)2;
}
//---------------------------------------------------------------------------

void __fastcall Tmainform::SpeedButton3Click(TObject *Sender) {

  mousestate = select;
  graphcanvas->Cursor = crIBeam;
}
//---------------------------------------------------------------------------



void __fastcall Tmainform::graphcanvasMouseDown(TObject *Sender,
      TMouseButton Button, TShiftState Shift, int X, int Y) {

  klik = true;
  startdrag = X;
}
//---------------------------------------------------------------------------

void __fastcall Tmainform::graphcanvasMouseUp(TObject *Sender, TMouseButton Button,
      TShiftState Shift, int X, int Y) {
  klik = false;

  switch (mousestate) {
    case zoom:
      if (startdrag < X) {
        graph.clear();
        graph.draw((graph.endsample / 600) * startdrag,(graph.endsample / 600) * X);
        graph.select(0,0);
      }
      else {
        graph.clear();
        graph.draw((graph.endsample / 600) * X,(graph.endsample / 600) * startdrag);
        graph.select(0,0);
      }
      zoomlevel->ItemIndex = 5;
    break;

    case select:
      if (startdrag - X == 0) {
        graph.select(0,0);
      }
    break;
  }
}
//---------------------------------------------------------------------------

void __fastcall Tmainform::graphcanvasMouseMove(TObject *Sender, TShiftState Shift,
      int X, int Y) {

  //Variabelen declareren
  int selection1, selection2;

  //Scrollen
  if (klik && mousestate == move) {
    if (X < startdrag && ((graph.startsample - scrollspeed) > 0)) {
      graph.clear();
      graph.draw(graph.startsample - scrollspeed,graph.endsample - scrollspeed);
      startdrag = X;
    }
    else if ((X > startdrag) && ((graph.endsample + scrollspeed) < wave1.num_samples)) {
      graph.clear();
      graph.draw(graph.startsample + scrollspeed,graph.endsample + scrollspeed);
      startdrag = X;
    }
  }

  //Selecteren
  if (klik && (mousestate == select || mousestate == zoom)) {

    selection1 = graph.startsample + (startdrag * ((graph.endsample - graph.startsample) / 600));
    selection2 = graph.startsample + (X * ((graph.endsample - graph.startsample) / 600));

    if (selection1 > selection2) {
      int hulp = selection2;
      selection2 = selection1;
      selection1 = hulp;
    }

    if (selection1 <= 0)
      selection1 = 0;

    graph.select(selection1,selection2);
  }
}
//---------------------------------------------------------------------------




void __fastcall Tmainform::zoomlevelChange(TObject *Sender) {

  if (zoomlevel->Text == "100%") {
  graph.clear();
  graph.draw(0,wave1.num_samples);
  }
  else if (zoomlevel->Text == "500%")
  {
  graph.clear();
  graph.draw(graph.startsample, (wave1.num_samples / 5) + graph.startsample);
  }
  else if (zoomlevel->Text == "1000%") {
  graph.clear();
  graph.draw(graph.startsample, (wave1.num_samples / 10) + graph.startsample);
  }
  else if (zoomlevel->Text == "1500%") {
  graph.clear();
  graph.draw(graph.startsample, (wave1.num_samples / 15) + graph.startsample);
  }
  else if (zoomlevel->Text == "2000%") {
  graph.clear();
  graph.draw(graph.startsample, (wave1.num_samples / 20) + graph.startsample);
  }
}
//---------------------------------------------------------------------------

void __fastcall Tmainform::Add_filterClick(TObject *Sender)
{
  filterform->ShowModal();
}
//---------------------------------------------------------------------------

void __fastcall Tmainform::Delete_filterClick(TObject *Sender)
{
  batch->Items->Delete(batch->ItemIndex);
}
//---------------------------------------------------------------------------

void __fastcall Tmainform::Button5Click(TObject *Sender)
{
  if (graph.endselection - graph.startselection > 5) {
    bliepdsp temp, temp1;

    wave1.cut(&temp,graph.startselection,graph.endselection);
    temp1 = temp;
    code_parser(batch,&temp,&temp1);
    wave1.paste(&temp1,graph.startselection);
  }
  else {
    code_parser(batch,&wave2,&wave1);
  }

  graph.clear();
  graph.draw(graph.startsample,graph.endsample);
}
//---------------------------------------------------------------------------

void __fastcall Tmainform::SaveFile1Click(TObject *Sender)
{
  SaveDialog1->Filter = "*.wav|*.wav";
  SaveDialog1->Execute();
  if (SaveDialog1->FileName != NULL)
    wave1.write_wav(SaveDialog1->FileName.c_str());
}
//---------------------------------------------------------------------------


void __fastcall Tmainform::Runbatch1Click(TObject *Sender)
{
  OpenDialog1->Filter = "*.ucf|*.ucf";
  OpenDialog1->Execute();
  if (OpenDialog1->FileName != NULL)
    batch->Items->LoadFromFile(OpenDialog1->FileName);
}
//---------------------------------------------------------------------------

void __fastcall Tmainform::Savebatchtofile1Click(TObject *Sender)
{
  SaveDialog1->Filter = "*.ucf|*.ucf";
  SaveDialog1->Execute();
  if (SaveDialog1->FileName != NULL)
    batch->Items->SaveToFile(SaveDialog1->FileName);
}
//---------------------------------------------------------------------------

void __fastcall Tmainform::Cut1Click(TObject *Sender)
{
  wave1 = wave2;
  wave1.cut(&wave2,graph.startselection,graph.endselection);
  graph.clear();
  graph.draw(graph.startsample,graph.endsample);
}
//---------------------------------------------------------------------------

void __fastcall Tmainform::Copy1Click(TObject *Sender)
{
  wave1 = wave2;
  wave1.copy(&wave2,graph.startselection,graph.endselection);
  graph.clear();
  graph.draw(graph.startsample,graph.endsample);
  graph.select(0,0);
}
//---------------------------------------------------------------------------

void __fastcall Tmainform::Past1Click(TObject *Sender)
{
  wave1.paste(&wave2,graph.startselection);
  graph.clear();
  graph.draw(graph.startsample,wave1.num_samples);
}
//---------------------------------------------------------------------------


void __fastcall Tmainform::btn_playClick(TObject *Sender)
{
  if (wave1.data != NULL)
  {
    if (bliep_query_paused()) {
      bliep_unpause();
    } else if (bliep_query_playing()) {
      bliep_pause();
    } else {
      bliep_play(&wave1);
    }
  }
}
//---------------------------------------------------------------------------


void __fastcall Tmainform::mainTimerTimer(TObject *Sender)
{
    char min[4];
    char sec[4];
  if(bliep_query_playing() && bliep_query_paused() != 1)
  {
    graph.playback(bliep_currentsample());
//    wave_to_graph(graph, &wave1, startdisplay, stopdisplay, 0, 0, bliep_currentsample());
    sprintf(min,"%02d",((bliep_currenttime() / 1000) / 60));
    sprintf(sec,"%02d",((bliep_currenttime() / 1000) % 60));
    label_position->Caption = AnsiString(min) + ":" + AnsiString(sec)  + " / " + wave1.get_duration();
  }
}
//---------------------------------------------------------------------------

void __fastcall Tmainform::btn_stopClick(TObject *Sender)
{
  if (bliep_query_playing()) {
    bliep_stop();
//    wave_to_graph(graph, &wave1, startdisplay, stopdisplay, 0, 0, 0);
    label_position->Caption = "00:00 / " + wave1.get_duration();
  }
}
//---------------------------------------------------------------------------

void __fastcall Tmainform::btn_pauseClick(TObject *Sender)
{
  if (wave1.data != NULL) {
    if (bliep_query_paused()) {
      bliep_unpause();
    } else if (bliep_query_playing()) {
      bliep_pause();
    }
  }
}
//---------------------------------------------------------------------------


void __fastcall Tmainform::FormCreate(TObject *Sender)
{
  graph.init(graphcanvas,Label1,Label2,Label3,Label4);        
}
//---------------------------------------------------------------------------

void __fastcall Tmainform::Settings1Click(TObject *Sender)
{
  settingsform->ShowModal();        
}
//---------------------------------------------------------------------------


void __fastcall Tmainform::Selectall1Click(TObject *Sender)
{
graph.select(0,wave1.num_samples);
}
//---------------------------------------------------------------------------


void __fastcall Tmainform::QuitClick(TObject *Sender)
{
Close();
}
//---------------------------------------------------------------------------




void __fastcall Tmainform::Image1Click(TObject *Sender)
{
  if (batch->ItemIndex > 0) {
    AnsiString temp;
    temp = batch->Items->Strings[batch->ItemIndex];
    batch->Items->Strings[batch->ItemIndex] = batch->Items->Strings[batch->ItemIndex - 1];
    batch->Items->Strings[batch->ItemIndex-1] = temp;
    batch->ItemIndex = batch->ItemIndex - 1;
  }
}
//---------------------------------------------------------------------------

void __fastcall Tmainform::Image2Click(TObject *Sender)
{
  if (batch->ItemIndex < batch->Items->Count-1) {
    AnsiString temp;
    temp = batch->Items->Strings[batch->ItemIndex];
    batch->Items->Strings[batch->ItemIndex] = batch->Items->Strings[batch->ItemIndex + 1];
    batch->Items->Strings[batch->ItemIndex+1] = temp;
    batch->ItemIndex = batch->ItemIndex + 1;
  }        
}
//---------------------------------------------------------------------------


void __fastcall Tmainform::About3Click(TObject *Sender)
{
  aboutform->Show();
}
//---------------------------------------------------------------------------


