#include "graph.h"
#include <math.h>
#include <stdio.h>
#include "bliepdsp.h"

#define GRAPH_WIDTH 600

graphformat::graphformat ()
:set(false),startselection(0),endselection(0) {
/* Constructor graphformat */

  bgcolor = (TColor) RGB(0,0,0);
  lblcolor = (TColor) RGB(50,50,50);
  txtcolor = (TColor) RGB(255,255,255);
  linecolor = (TColor) RGB(255,255,255);
  samplecolor = (TColor) RGB(060,179,113);
  selectioncolor = (TColor) RGB(40,40,40);
  playbackcolor = (TColor) RGB(255,255,255);
}

void graphformat::init (TImage *graph_in, TLabel *label1, TLabel *label2, TLabel *label3, TLabel *label4) {
/* pre: -
   post: De canvas van *graph wordt als tekenveld ingesteld en de labels zullen dienen voor de tijdsaanduiding
   Width: 600, Height: 115 en lege grafiek opgemaakt */

  //Adres van te gebruiken componenten opslaan
  graph = graph_in;
  Label1 = label1;
  Label2 = label2;
  Label3 = label3;
  Label4 = label4;

  clear();
}

void graphformat::data (bliepdsp *wave_in) {
/* pre: -
   post: *wave_in wordt geregistreerd zijnde grafiekdata */

  wave = wave_in;
  startsample = 0;
  endsample = wave->num_samples;
  set = true;
}

void graphformat::draw (unsigned int start, unsigned int end) {
/* pre: init() en data() minimaal 1x aangeroepen
   post: Samples uit *wave van start tot end op canvas van *graph */

  if (set == false)
    return;

  //Declareren van variabelen (uitleg volgt in code)
  unsigned int samplecount, sampledeler, xinterval, x;
  short int buffer16;
  char buffer8, *source;
  float xhulp;

  //Foutcontrole
  if (end > wave->num_samples)
    end = wave->num_samples;

  //Als aantal samples > GRAPH_WIDTH dan samples weglaten
  //Als aantal samples < GRAPH_WIDTH samples gelijkmatig over graph spreiden
  samplecount = (end - start) + 1;
  if (samplecount >= GRAPH_WIDTH) {
    sampledeler = (int)((samplecount / 600) + 0.5);  //Intervalgroote tussen samples
    xinterval = 1;                                   //Intervalgroote voor de x-as
  }
  else {
    sampledeler = 1;                                 //Intervalgroote tussen samples
    xhulp = ((600 / samplecount) + 0.5);
    xinterval = (int) xhulp;                         //Intervalgroote voor de x-as
    end = end + ((xhulp - xinterval) * GRAPH_WIDTH);
    if (end > wave->num_samples)
      end = wave->num_samples;
  }

  //Opslaan voor later gebruik
  startsample = start;
  endsample = end;

    //Horizontale lijn(en)
    if (wave->header.num_channels == 1) {
      graph->Canvas->Pen->Color = linecolor;
      graph->Canvas->Pen->Width = 1;
      graph->Canvas->MoveTo(0,102);
      graph->Canvas->LineTo(600,102);
    }
    else if (wave->header.num_channels == 2) {
      graph->Canvas->Pen->Color = linecolor;
      graph->Canvas->Pen->Width = 1;
      graph->Canvas->MoveTo(0,50);
      graph->Canvas->LineTo(600,50);
      graph->Canvas->MoveTo(0,154);
      graph->Canvas->LineTo(600,154);
    }


  //samplecolor instellen etc.
  x = 0;
  graph->Canvas->Pen->Color = samplecolor;

  //Teken het hele gebeuren
  switch (wave->header.num_channels) {
    case 1:
      graph->Canvas->MoveTo(0,50);
      if (wave->header.bits_per_sample == 8) {
        for (unsigned int i=startsample; i<endsample-1; i+=sampledeler) {
          x+=xinterval;
          source = wave->data + i;
          memcpy(&buffer8,source,1);
          buffer8 = (unsigned int) buffer8 / 2.55;
          graph->Canvas->LineTo(x,((100 - buffer8)+52)); //+52 om bij nul-lijn te komen, 100 - .. om spiegeling te voorkomen
        }
      } else
      if (wave->header.bits_per_sample == 16) {
        for (unsigned int i=startsample; i<endsample-1; i+=sampledeler) {
          x+=xinterval;
          source = wave->data + (i*2);
          memcpy(&buffer16,source,2);
          buffer16 = (buffer16 + 32768) / 655.36;
          graph->Canvas->LineTo(x,(100 - buffer16)+52);
        }
      }
    break;
    case 2:
      graph->Canvas->MoveTo(0,50);
      if (wave->header.bits_per_sample == 8) {
        for(unsigned int i=startsample; i<endsample-1; i+=sampledeler) {
          x+=xinterval;
          source = wave->data + (i * wave->header.num_channels);
          memcpy(&buffer8,source,1);
          buffer8 = (unsigned int) buffer8 / 2.55;
          graph->Canvas->LineTo(x,100 - buffer8);
        }
        graph->Canvas->MoveTo(0,105);
        x=0;
        for (unsigned int i=startsample; i<endsample-1; i+=sampledeler) {
          x+=xinterval;
          source = wave->data + ((i * wave->header.num_channels) + 1);
          memcpy(&buffer8,source,1);
          buffer8 = (unsigned int) buffer8 / 2.55;
          graph->Canvas->LineTo(x,(100 - buffer8)+105);
        }
      } else
        if (wave->header.bits_per_sample == 16) {
          for (unsigned int i=startsample; i<endsample-1; i+=sampledeler) {
            x+=xinterval;
            source = wave->data + (i * wave->header.num_channels * 2);
            memcpy(&buffer16,source,2);
            buffer16 = (buffer16 + 32768) / 655.36;
            graph->Canvas->LineTo(x,100 - buffer16);
          }
          graph->Canvas->MoveTo(0,105);
          x=0;
          for (unsigned int i=startsample; i<endsample-1; i+=sampledeler) {
            x+=xinterval;
            source = wave->data + ((i * wave->header.num_channels * 2) + 1);
            memcpy(&buffer16,source,2);
            buffer16 = (buffer16 + 32768) / 655.36;
            graph->Canvas->LineTo(x,(100 - buffer16)+105);
          }
        }
    break;
  }

  //Seconde aanduiding bijwerken
  update_time_indication();
}

void graphformat::select (unsigned int start, unsigned int end) {
/* pre: init() data() en draw() zijn minimaal 1x aangeroepen
   post: selectie wordt gemaakt */

  if (set == false)
    return;

  //Declareren van variabelen
  int samplecount, sampledeler, x, xinterval, xverschil;
  float xhulp;

  //Selectie vastleggen
  startselection = start;
  endselection = end;

  //Selectie verwijderen
  if (startselection == 0 && endselection == 0) {
    clear();
    draw(startsample,endsample);
    return;
  }

  //Clear grafiek
  clear();

  x=0;
  //Als aantal samples > GRAPH_WIDTH dan samples weglaten
  //Als aantal samples < GRAPH_WIDTH samples gelijkmatig over graph spreiden
  samplecount = (endsample - startsample) + 1;
  if (samplecount >= GRAPH_WIDTH) {
    sampledeler = (int)((samplecount / 600) + 0.5);  //Intervalgroote tussen samples
    xinterval = 1;                                   //Intervalgroote voor de x-as
  }
  else {
    sampledeler = 1;                                 //Intervalgroote tussen samples
    xhulp = ((600 / samplecount) + 0.5);
    xinterval = (int) xhulp;                         //Intervalgroote voor de x-as
    endsample = endsample + ((xhulp - xinterval) * GRAPH_WIDTH);
    if (endsample > wave->num_samples)
      endsample = wave->num_samples;
  }

  graph->Canvas->Brush->Color = selectioncolor;
  graph->Canvas->MoveTo(0,50);
  for (unsigned int i=startsample; i<endsample-1; i+=sampledeler) {
    x+=xinterval;
    graph->Canvas->MoveTo(x,50);
    if (i >= startselection && i <=endselection)
      graph->Canvas->FillRect(Rect(x,0,x+1,204));
  }

  //Herteken grafiek
  draw(startsample,endsample);
}

void graphformat::playback (unsigned int currentsample) {
/* pre: init() data() en draw() zijn minimaal 1x aangeroepen
   post: currentsample wordt met verticale lijn aangegeven */

  //Declareren van variabelen
  int samplecount, sampledeler, x, xinterval, xverschil;
  float xhulp;
  bool line_set = false;

  //Clear grafiek
  clear();

  x=0;
  //Als aantal samples > GRAPH_WIDTH dan samples weglaten
  //Als aantal samples < GRAPH_WIDTH samples gelijkmatig over graph spreiden
  samplecount = (endsample - startsample) + 1;
  if (samplecount >= GRAPH_WIDTH) {
    sampledeler = (int)((samplecount / 600) + 0.5);  //Intervalgroote tussen samples
    xinterval = 1;                                   //Intervalgroote voor de x-as
  }
  else {
    sampledeler = 1;                                 //Intervalgroote tussen samples
    xhulp = ((600 / samplecount) + 0.5);
    xinterval = (int) xhulp;                         //Intervalgroote voor de x-as
    endsample = endsample + ((xhulp - xinterval) * GRAPH_WIDTH);
    if (endsample > wave->num_samples)
      endsample = wave->num_samples;
  }

  graph->Canvas->Pen->Color = playbackcolor;
  graph->Canvas->MoveTo(0,50);
  for (unsigned int i=startsample; i<endsample-1; i+=sampledeler) {
    x+=xinterval;
    graph->Canvas->MoveTo(x,50);
    if (i > currentsample && line_set == false) {
      graph->Canvas->MoveTo(x,0);
      graph->Canvas->LineTo(x,204);
      line_set = true;
    }
  }

  //Herteken grafiek
  draw(startsample,endsample);

}

void graphformat::clear () {
/* pre: init() minimaal 1x aangeroepen
   post: Width: 600, Height: 115 en lege grafiek opgemaakt */

  //Afmetingen graph
  graph->Width = GRAPH_WIDTH;
  graph->Height = 220; //maximaal 100 px per channel, 16 px voor sec aanduiding en 4px marge

  //Canvas tussenstap overbodig maken
  TCanvas *graph1;
  graph1 = (*graph).Canvas;

  //Achtergrond
  graph1->Brush->Color = bgcolor;
  graph1->FillRect(Rect(0,0,600,205));
  graph1->Brush->Color = lblcolor;
  graph1->FillRect(Rect(0,204,600,220));
}

void graphformat::update_time_indication() {
/* pre: init() en data() zijn minimaal 1x aangeroepen
   post: tijd indicatie is bijgewerkt. */

  //Kleuren aanpassen
  Label1->Color = this->lblcolor;
  Label2->Color = this->lblcolor;
  Label3->Color = this->lblcolor;
  Label4->Color = this->lblcolor;
  Label1->Font->Color = this->txtcolor;
  Label2->Font->Color = this->txtcolor;
  Label3->Font->Color = this->txtcolor;
  Label4->Font->Color = this->txtcolor;

  char buffer[100];
  // Linker nummer
  sprintf(buffer,"%0.3f", (float) ((float)startsample / wave->header.sample_rate));
  Label1->Caption = AnsiString(buffer);

  // Rechter nummer
  sprintf(buffer,"%0.3f", (float) ((float)endsample / wave->header.sample_rate));
  Label2->Caption = AnsiString(buffer);

  // Midden links
  sprintf(buffer,"%0.3f", (float) (((float)(endsample - startsample) / 3 + startsample) / wave->header.sample_rate));
  Label3->Caption = AnsiString(buffer);

  // Midden rechts
  sprintf(buffer,"%0.3f", (float) ((((float)(endsample - startsample) / 3) * 2 + startsample) / wave->header.sample_rate));
  Label4->Caption = AnsiString(buffer);
}
